"""
############################################################################
##  buildings.py
##  
## Contains different builidng classes. Buildings sit on lots and are 
## resposible for creating units, generating money, and ground and air defence
## The global dictionary 'buildings_class_map' is map from building names 
## to their classes
############################################################################
"""
import  os
from    Opioid2D        import *
from    global_options  import game_options
from    targetables     import Targetable
from    units           import Tank, Infantry
from    utils           import find_closest_n_enemy_units


class Building(Sprite, Targetable):
    """ 
        #######################################################################
        ## This is the superclass of all buildings. 
        ## 
        ## Object Variables:
        ##  name        Name of this building
        ##  max_health  Maximum number of hitpoints it cab take befire dying
        ##  maint_chrg  The amount of Maintenance charged eahc money cycle
        ##  side        What side are we on ('left' or 'right')
        ##  group_name  The group we belong to: e.g. live_buildings_[side] 
        ##  status      Current status (eg.'ready', 'under-const')
        ##  parent_lot  The lot that we are sitting on
        ##  act_delay   Number of miliseconds between actions 
        ##              eg. how often barak builds troops or refinery
        ##              generates money. A None value means building doesn't 
        ##              have a periodical action
        #######################################################################
    """
    layer       = "buildings"
    
    
    def __init__(self, name, max_health, maintence_chrg, act_delay, parent_lot):
        """
            Constructor
        """
        Sprite.__init__(self)
        Targetable.__init__(self, max_health)
        self.name               = name
        self.maintence_chrg = maintence_chrg
        self.parent_lot         = parent_lot
        self.act_delay          = act_delay
        self.side               = self.parent_lot.owner_player.side
        self.group_name         = "live_buildings_" + self.side
        self.join_group(self.group_name)

        # Initialize the periodical call of take_action()
        # When act_delay is None, we don't have a repeating action
        if self.act_delay is not None:
            act_delay_secs  = float(self.act_delay) / 1000.0
            call_action     = Delay(act_delay_secs) + CallFunc(self.take_action)
            repeat_call     = Repeat(call_action)
            self.do(repeat_call)
        
        # Initialize the buildings image
        image_path =    game_options.gfx_path + self.name + \
                        "_" + self.side + ".png"
        self.image = image_path
        self.set(   scale       = game_options.building_scale, 
                    position    = self.parent_lot.get_position())
        
        # Update player's money-growth rate
        self.parent_lot.owner_player.update_money_growth()
    
    def take_action(self):
        """
            This method is automatically called every act-delay period.
            Whatever action the building takes periodaically (e.g. barak
            building units or refinery generating money) should happen in here
        """
        print "this method should be overridden by subclasses"
    
    def build(self):
        """
            Called when the building is built. Shows a nice animation
        """
        # TODO: Implement Pretty Animation
        pass

    def sell(self):
        """
            Called when the building is sold. Removes building from player's
            live_buildings group and Shows a nice animation
        """
        leave_group(self.group_name)
        self.parent_lot.building = None
        self.parent_lot.owner_player.update_money_growth()
    
    def die(self):
        """
            Called when the building is destroyed. Removes building from player's
            live_buildings group and deletes the object
        """
        self.leave_group(self.group_name)
        self.parent_lot.building = None
        self.do (    AlphaFade(0,secs=0.5) + Delete )
        self.parent_lot.owner_player.update_money_growth()
        
        
    def _show_anim_hit(self):
        """
            Shows an animation when the building is hit.
        """
        # TODO: Implement Pretty Animation
        pass
    
    def _show_anim_die(self):
        """
            Shows an animation when the building is destroyed
        """
        # TODO: Implement Pretty Animation
        pass    

class Barak(Building):
    """ 
        #######################################################################
        ## Barak is a building that builds Infantry
        #######################################################################
    """

    def __init__(self, parent_lot):
        """
            Constructor
        """
        Building.__init__(  
            self, 
            name            = 'barak', 
            max_health      = game_options.buildings['barak']['max_health'],
            maintence_chrg  = game_options.buildings['barak']['maintence_chrg'],
            act_delay       = game_options.buildings['barak']['action_delay'],
            parent_lot      = parent_lot) 

        
    def take_action(self):
        """
            This method is automatically called every act-delay period.
            Barak builds one Infintary every time this function is called
            NOTE: Do not call this method manually! This is only to be called
            by the timer
        """
        # Build a new Infantry
        if  (   self.parent_lot.owner_player.money < 
                game_options.units['infantry']['price']):
            # Can't afford a tank
            return
        self.parent_lot.owner_player.money = \
            self.parent_lot.owner_player.money -  \
            game_options.units['infantry']['price']

        new_infantry = Infantry(self.parent_lot.owner_player)
        # Initilize its position
        new_infantry.set(position = self.get_position())
        if self.side == 'left':
            new_infantry.rect.left = self.rect.right
        else:
            new_infantry.rect.right = self.rect.left


class Factory(Building):
    """ 
        #######################################################################
        ## Factory is a building that builds Tank
        #######################################################################
    """

    def __init__(self, parent_lot):
        """
            Constructor
        """
        Building.__init__(  
            self, 
            name            = 'factory', 
            max_health      = game_options.buildings['factory']['max_health'],
            maintence_chrg  = game_options.buildings['factory']['maintence_chrg'],
            act_delay       = game_options.buildings['factory']['action_delay'],
            parent_lot      = parent_lot) 
        
##        #### DEBUG ####
##        self.has_one_tank = False
##        #### /DEBUG ####

    def take_action(self):
        """
            This method is automatically called every act-delay period.
            Factory builds one Tank every time this function is called
            NOTE: Do not call this method manually! This is only to be called
            by the timer
        """
##        #### DEBUG ####
##        if(self.has_one_tank):
##            return       
##        self.has_one_tank = True
##        #### /DEBUG ####
        
        # Build a new Tank
        if  (   self.parent_lot.owner_player.money < 
                game_options.units['tank']['price']):
            # Can't afford a tank
            return
        self.parent_lot.owner_player.money = \
            self.parent_lot.owner_player.money -  \
            game_options.units['tank']['price']
        
        new_tank = Tank(self.parent_lot.owner_player)

        # Initilize its position
        new_tank.set(position = self.get_position())
        if self.side == 'left':
            new_tank.rect.left = self.rect.right
        else:
            new_tank.rect.right = self.rect.left
        

class Refinery(Building):
    """ 
        #######################################################################
        ## Refinery is a building that generates Money
        ##  Note that a refinary has no periodic action. All it does is update
        ##  player's money_growth whenevr it is built ot destroyed
        #######################################################################
    """

    def __init__(self, parent_lot):
        """
            Constructor
        """
        Building.__init__(  
            self, 
            name            = 'refinery', 
            max_health      = game_options.buildings['refinery']['max_health'],
            maintence_chrg  = game_options.buildings['refinery']['maintence_chrg'],
            act_delay       = game_options.buildings['refinery']['action_delay'],
            parent_lot      = parent_lot) 
            
        
    def sell(self):
        """
            Called when the building is sold. Update player's money-growth rate
        """
        self.parent_lot.owner_player.update_money_growth()
        Building.sell(self)
    


class Turret(Building):
    """ 
        #######################################################################
        ## Turret is a building that defends the base.
        ## It attacks teh surrounding enemy units every action_delay period
        #######################################################################
    """

    def __init__(self, parent_lot):
        """
            Constructor
        """
        Building.__init__(  
            self, 
            name            = 'turret', 
            max_health      = game_options.buildings['turret']['max_health'],
            maintence_chrg  = game_options.buildings['turret']['maintence_chrg'],
            act_delay       = game_options.buildings['turret']['action_delay'],
            parent_lot      = parent_lot) 
        self.hitpoint   = game_options.buildings['turret']['hitpoint']
        self.fire_range = game_options.buildings['turret']['fire_range']
        self.num_targets= game_options.buildings['turret']['num_targets']

    def take_action(self):
        """
            This method is automatically called every act-delay period.
            Once called, turret attacks some of the surrounding enemy units
        """
        # find the closest num_targets enemy units
        targets = find_closest_n_enemy_units(   self, self.fire_range, 
                                                self.num_targets)
        n = len(targets)
        if n < 1:
            return  # no one to kill :(
              
        # hit the targets
        ind_hitpoint = float(self.hitpoint) / float(n)  
        for target in targets:
            target.get_hit(ind_hitpoint)
        
        # show a pretty animation
        self._show_anim_attack()

    def _show_anim_attack(self):
        """
            Shows an animation when the unit is attacking.
        """
        image_pre = game_options.gfx_path + self.name + "_" + self.side
        attack_anim =  [image_pre + "_atk_1.png", 
                        image_pre + "_atk_2.png", 
                        image_pre + "_atk_3.png", 
                        image_pre + ".png"]
        self.do(Animate(attack_anim, secs=0.4, mode=StopMode))
        

buildings_class_map             = {}    # Maps from building names to classes
buildings_class_map['barak']    = Barak
buildings_class_map['factory']  = Factory
buildings_class_map['refinery'] = Refinery
buildings_class_map['turret']   = Turret

 
