"""
############################################################################
##  buttons.py
##  
##  Defines the panel buttons
## 
############################################################################
"""
from    Opioid2D        import *
from    global_options  import game_options


class Button(gui.GUISprite):
    """ 
        #######################################################################
        ## Superclass of all button classes. Has a border and an icon, and  
        ## responds to mouse events
        ##
        ## Object Variables:        
        ##  icon            # holds the actual button image/icon
        ##  parent_panel    # the panel where this button sits on
        ##  enabled         # When True, button can be clicked
        ##  visible         # When truem button is visible
        #######################################################################
    """
    layer   = 'gui_buttons'   
    #the guisprite's image is just its border
    image   = game_options.gfx_path + "button_border_out.png" 
    
    def __init__(self, icon_image, parent_panel):
        """
            Constructor
            icon_image is the image path for the button icon
        """
        gui.GUISprite.__init__(self)
        # Button.image contains the button's border. Button.icon.image contains
        # the actual image of this button
        self.icon  = gui.GUISprite(icon_image)
        self.icon.on_enter  = self.on_enter
        self.icon.on_exit   = self.on_exit
        self.icon.on_press  = self.on_press
        self.icon.on_release= self.on_release
        self.icon.on_click  = self.on_click
        
        self.icon.set(  layer       = "gui_buttons", 
                        position    = self.get_position(),
                        color       = game_options.button_colors['normal'])
        self.icon.attach_to(self)
        self.parent_panel       = parent_panel
        self.enabled            = True
        self.visible            = True

    def set_enabled(self, enabled):
        """
            Sets the button enabled or disabled
        """
        self.enabled = enabled
        if self.enabled  is True:
            self.icon.set(color = game_options.button_colors['normal'])
        else:
            self.icon.set(color = game_options.button_colors['disabled'])

    def on_enter(self):
        """
            called when the mouse cursor enters the sprite's area
        """
        if self.visible and self.enabled:
            self.icon.set(color = game_options.button_colors['highlight'])
        if self.visible:
            self.parent_panel.on_hover_enter(self.building_name)
        
    def on_exit(self):
        """
            called when the mouse cursor exits the sprite
        """
        if self.visible and self.enabled:
            self.icon.set(color = game_options.button_colors['normal'])
        if self.visible:
            self.parent_panel.on_hover_exit(self.building_name)
        
    def on_press(self):
        """
            called when the user presses the mouse button down on this sprite
        """
        if self.visible and self.enabled:
            self.image = game_options.gfx_path + "button_border_in.png" 
    
    def on_release(self):
        """
            called when the user releases the mouse button OR moves the cursor
            out of this sprite
        """
        if self.visible and self.enabled:
            self.image = game_options.gfx_path + "button_border_out.png"
    
    def on_click(self):
        """
            called when the user presses and releases the mouse button on this
            sprite
        """
        # Subclasses should override Button.on_click()"
        pass
    
    def show(self):
        """
            make this button visible
        """
        self.set(alpha = 1.0)
        self.icon.set(alpha = 1.0)
        self.visible = True

    def hide(self):
        """
            make this button invisible
        """
        self.set(alpha = 0.0)
        self.icon.set(alpha = 0.0)
        self.visible = False

class BuyButton(Button):
    """ 
        #######################################################################
        ## This is the button the player clicks to buy a building
        ##
        ## Object Variables:
        ##  building_name   # Name of the building we are selling
        #######################################################################
    """
    def __init__(self, building_name, parent_panel):
        """ 
            Constructor
            building_name is the name of the building being sold
        """
        icon_image = game_options.gfx_path + building_name + "_icon.png"
        Button.__init__(self, icon_image, parent_panel)
        self.building_name  = building_name


    def on_click(self):
        """
            Called when user clicks on the buy button
        """
        self.parent_panel.lot.buy(self.building_name)