"""
############################################################################
##  lots.py
##  
## Contains the Lot class, plus tools to initilize lots on the screen
############################################################################
"""
from    Opioid2D        import *
from    global_options  import game_options
from    utils           import relative_pos_to_side
from    buildings       import buildings_class_map

all_lots = []   # A list of aall lots created. used for lot selection clearing

class Lot(gui.GUISprite):
    """ 
        #######################################################################
        ## A lot is the place holder for a building. It is derived from GUISrite 
        ## so that it responds to mouse events. It also takes care of the logic
        ## for buying and selling buildings
        ##
        ## Object Variables:
        ##  building        # The building on this lot. None means empty lot
        ##  owner_player    # The player who owns this lot
        ##  is_locked       # When True, user cannot manipulate lot by mouse
        ##  is_selected     # True when this lot is selected
        #######################################################################
    """
    layer           = "lots"
    image           = game_options.gfx_path + "lot.png"
   
    
    def __init__(self, player, is_locked):
        """
            Constructor
        """
        self.owner_player   = player
        self.is_locked      = is_locked
        self.building       = None
        self.is_selected    = False
        
        gui.GUISprite.__init__(self)
        side = self.owner_player.side
        self.set(   scale = game_options.building_scale,
                    alpha = game_options.lot_alpha,
                    color = game_options.player[side]['color']['normal'])
        
        # add it to global list of lots
        all_lots.append(self)

    def on_enter(self):
        """
            called when the mouse cursor enters the sprite's area
        """
        if not self.is_selected and not self.is_locked:
            side = self.owner_player.side
            self.set(   color=game_options.player[side]['color']['highlight'],
                        alpha=game_options.lot_alpha)
        
    def on_exit(self):
        """
            called when the mouse cursor exits the sprite
        """
        if not self.is_selected and not self.is_locked:
            side = self.owner_player.side
            self.set(   color=game_options.player[side]['color']['normal'],
                        alpha=game_options.lot_alpha)
    
    def on_click(self):
        """
            Called when user clicks on this lot
            Shows either the BuyPanel or the ControlPanel
        """
        if not self.is_selected and not self.is_locked:
            panel_manager = Director.scene.panel_manager
            self.select()
        
    def select(self):
        """
            Selects the current lot
        """
        # Unselect all lots
        reset_selections()
        # select this lot
        self.is_selected = True    
        # change the lot's color
        side = self.owner_player.side
        self.set(   color=game_options.player[side]['color']['selected'],
                    alpha=game_options.lot_alpha)
        
        # show the proper panel
        panel_manager = Director.scene.panel_manager
        if self.building is None:
            # when no building, show the buy-panel
            panel_manager.set_current_panel('buy_panel', 
                                            owner_player=self.owner_player,
                                            owner_lot=self)
        else:
            # When there is a building, show the control-panel
            # TODO: Implement Control Panel
            print "todo!"
        
    def unselect(self):
        """
            UnSelects the current lot
        """
        self.is_selected = False    
        # change the lot's color
        side = self.owner_player.side
        self.set(   color=game_options.player[side]['color']['normal'],
                    alpha=game_options.lot_alpha)

    def buy(self, building_name):
        """
            Buys the specified building and builds it in this lot
        """
        # Check that the lot is empty
        if self.building is not None:
            print "Cannot buy a building in a non-empty lot"
            return
        # See if player can afford it
        if  (   self.owner_player.money < 
                game_options.buildings[building_name]['buy_price']):
            #TODO: Implement a warning system
            print "you cannot afford this building"
            return
        # Charge for the building
        self.owner_player.money = self.owner_player.money - \
                                game_options.buildings[building_name]['buy_price']
        # Buy the building
        building_class  = buildings_class_map[building_name]
        self.building   = building_class(self)

        # Bring back the main panel
        panel_manager = Director.scene.panel_manager
        panel_manager.set_current_panel('main_panel')
        # Unselect the lot
        self.unselect()

def init_lots(player, is_locked):
    """
        #######################################################################
        ## Creates a list of lots for the player, and initilize their locations
        #######################################################################
    """
    player_lots = []
    for lot_num in game_options.lot_pos:
        new_lot = Lot(player, is_locked)
        lot_position = relative_pos_to_side(game_options.lot_pos[lot_num], 
                                            player.side)
        new_lot.set(position = lot_position)
        player_lots.append(new_lot)
    player.set_lots(player_lots)


def reset_selections():
    """
        Unselects all lots created so far.
        This metod is called when user selects a new lot (or another object)
        To clear the section ofany previous lot
    """
    for lot in all_lots:
        lot.unselect()
