"""
############################################################################
##  panels.py
##  
## Contains classes resposible for the main panel.
## 
############################################################################
"""
from    Opioid2D        import *
from    global_options  import game_options
from    buttons         import BuyButton


class PanelManager:
    """ 
        #######################################################################
        ## This class initilizes a list of panels, and sets the currently
        ## visible panel 
        ##
        ## Object Variables:
        ##      panels          # A dictionary of form:
        ##                      #   [panel name] -> panel object
        ##                      #   There are 2 special panels: 
        ##                      #   'main_panel' and 'buy_panel'
        ##      current_panel   # Holds the name of currently visible panel 
        #######################################################################
    """
    
    def __init__(self):
        """
            Constructor
        """
        # Initilize the special panels
        self.current_panel          = None
        self.panels                 = {}
        self.panels['main_panel']   = MainPanel()
        self.panels['buy_panel']    = BuyPanel()
        # Hide all panels
        for panel in self.panels:
            self.panels[panel].hide()

    def set_current_panel(self, panel_name, owner_player=None, owner_lot=None):
        """
            Changes the current panel. Also changes the owner of the 
            newly visible panel
        """
        # Hide the old panel
        if self.current_panel is not None:
            self.current_panel.hide() 
        # Change the current panel
        self.current_panel = self.panels[panel_name]
        # Make the new panel visible
        self.current_panel.show()
        # change the owner
        self.current_panel.set_player(owner_player)
        self.current_panel.set_lot(owner_lot)

    def get_current_panel(self):
        """
            Returns the currently visible panel
        """
        return self.current_panel
    
class Panel(Sprite):
    """ 
        #######################################################################
        ## All panels are a subclass of this class
        ##
        ## Object Variables:
        ##      player      # The player that owns this panel
        ##      lot         # The lot that owns this panel
        #######################################################################
    """
    layer       = "gui_panels"
    image       = game_options.gfx_path + "main_panel.png"

    def __init__(self):
        """
            Constructor
        """
        Sprite.__init__(self)
        self.set(position = (   game_options.win_width / 2,
                                game_options.panel_height / 2))
        
    def set_player(self, player):
        """ 
            Sets the player that owns this panel
        """
        self.player = player

    def set_lot(self, lot):
        """ 
            Sets the lot that owns this panel
        """
        self.lot = lot
        
    def on_hover_enter(self, object_name):
        """
            This method is called when the user hovers the mouse over one of
            this panel's objects. 
        """

    def on_hover_exit(self, object_name):
        """
            This method is called when the user exits hovering the mouse over 
            one of this panel's objects. 
        """
    
    def show(self):
        """
            make this panel and all its objects visible
        """
        self.set(alpha = 1.0)
 
    def hide(self):
        """
            make this panel and all its objects invisible
        """
        self.set(alpha = 0.0)
    

class MainPanel(Panel):
    """ 
        #######################################################################
        ## Represents the main panel. 
        ## Currently this just shows pretty picture
        #######################################################################
    """
    pass
    
class BuyPanel(Panel):
    """ 
        #######################################################################
        ## When user clicks on an empty lot, this panel is shown
        ## It shows a list of available buildings that user can buy
        ## If a building is not available (i.e. the user has reached his limit)
        ## then it will be shown as a disbaled button
        ##
        ##  Object Variables:
        ##   buy_buttons # A dictionary of [building-name] -> BuyButton
        ##   desc_texts  # A dictionary of [building-name] -> Description Text
        #######################################################################
    """
    
    def __init__(self):
        """ 
            Constructor
        """
        Panel.__init__(self)
        # Create the list of buttons
        self.buy_buttons = {}
        self.__init_buttons()
        # Initilize the 
        self.desc_texts = {}
        self.__init_desc_texts()
        
    def __init_buttons(self):
        """
            A private method, used by the constructor to create and place the 
            buy-buttons
        """
        left_dist       =   game_options.buy_button['dist_left']
        bottom_dist     =   game_options.buy_button['dist_bottom']
        button_distance =   game_options.buy_button['dist_between']
        btn_counter     = 0
        for building in game_options.buildings:
            # build the button
            new_button = BuyButton(building, parent_panel = self)  
            # calculate its position
            panel_bottom_left = self.rect.bottomleft
            new_x = panel_bottom_left[0] + left_dist
            new_y = panel_bottom_left[1] - bottom_dist
            new_button.rect.bottomleft = (new_x, new_y)
            # add it to our list of buttons
            self.buy_buttons[building] = (new_button)
            btn_counter = btn_counter + 1
            # update the distance for next button
            left_dist = left_dist +  button_distance
    
    def __init_desc_texts(self):
        """
            A private method, used by the constructor to initilize the 
            description text sprites. These are shown once the user hovers the
            mouse pointer on a building.
        """
        for building in game_options.buildings:
            font = BitmapFont(game_options.font_path + "font_small.o2f")
            text = game_options.buildings[building]['discription'] \
                    % (game_options.buildings[building]['buy_price'])
            text_sprite = font.create(text, single=True)
            text_sprite.rect.topright = self.rect.topright
            text_sprite.set(layer='gui_text')  
            # TODO: Find a better way to make text sprides invisible
            text_sprite.set(alpha = 0.0)  # make it invisible
            self.desc_texts[building] = text_sprite
        
        
    def on_hover_enter(self, object_name):
        """
            This method is called when the user hovers the mouse on one of
            our buttons. The button_name is the nameof the building which
            the button represents. We use this method to show the description 
            text
        """
        self.desc_texts[object_name].set(alpha = 1.0)
        
    def on_hover_exit(self, object_name):
        """
            This method is called when the user exits hovering the mouse over 
            one of this panel's objects. 
        """
        self.desc_texts[object_name].set(alpha = 0.0)

    def show(self):
        """
            make this panel and all its objects visible
        """
        Panel.show(self)
        # show the buttons
        for button in self.buy_buttons:
            self.buy_buttons[button].show()
    
    def hide(self):
        """
            make this panel and all its objects invisible
        """
        Panel.hide(self)
        # hide the buttons
        for button in self.buy_buttons:
            self.buy_buttons[button].hide()
        # hide the description texts
        for desc_text in self.desc_texts:
            self.desc_texts[desc_text].set(alpha = 0.0)
