"""
############################################################################
##  players.py
##  
## Contains 3 classes: 
##      - Player: The superclass of all player classes
##      - GUIPlayer: class representing the GUI player
##      - AIPlayer: class representing the AI player
## 
############################################################################
"""
from    Opioid2D        import *
from    Queue           import Queue
from    global_options  import game_options

class Player(Sprite):
    """ 
        #######################################################################
        ## To carry player actions, the game ask the Player object of each side
        ## for the next action. Class player is the superclass of all player 
        ## classes
        ## 
        ## Note that Player objects are not really sprites. The only reason 
        ## that the Player class is a subclass of sprite is to get the delayed
        ## call back actions, for updating money.
        ##
        ## Properties:
        ##  money           # The amount of money the player has (property)
        ##  money_growth    # The amount of money player recieves per cycle
        ## 
        ## Object Variables:        
        ##  __money         # Private variables that holds the actual money 
        ##  __money_growth  # Above is a property, this is the private value
        ##  side            # Player's Side (can be 'left' or 'right')
        ##  lots            # List of Lots objects, 
        ##                  # which contain a building or are empty
        #######################################################################
    """

    def __init__(self, side, init_money):
        """
            Constructor 
        """
        Sprite.__init__(self)
        self.side           = side
        self.__money        = init_money
        self.__money_growth = 0
        self.lots           = []
        
        # Initialize the periodical call to update money
        act_delay_secs  = float(game_options.pay_cycle_delay) / 1000.0
        call_action     = Delay(act_delay_secs) + CallFunc(self.update_money)
        repeat_call     = Repeat(call_action)
        self.do(repeat_call)

    def get_action(self, other_player):
        """
            Based on the status of me and the other player, chooses an action
            The possible actions are buying new buildings in empty lots or 
            selling a current building
            The game periodically calls this method for every player 
        """
        pass

    def set_lots(self, new_lots):
        """
            Set player's lots
        """
        self.lots = new_lots
                
    def update_money(self):
        """
            This method is called once per game cycle and 
            based on the money_growth amount, updates players money
        """
        if self.__money_growth != 0:
            self.__money = self.__money + self.__money_growth
            Director.scene.status_bar.update()
        # TODO: if money is negative (can't pay dues) reduce the health of 
        # buildings with positive maintenace
    ## Define the properties
    def set_money(self, new_money):
        """
            sets the player's money, and updates the statusbar
        """
        self.__money = new_money
        Director.scene.status_bar.update()

    def get_money(self):
        """
           Return's player's money
        """
        return(self.__money)

    def set_money_growth(self, new_money_growth):
        """
            sets the player's money_growth, and updates the statusbar
        """
        self.__money_growth = new_money_growth
        Director.scene.status_bar.update()
        
    def get_money_growth(self):
        """
           Return's player's money_growth
        """
        return(self.__money_growth)

    def update_money_growth (self):
        """
            This method is called whenever the player buys or loses
            a refinary. It counts the number of player's refinaries
            and number of building swith positive maintenance cost
            and calculates the player's money_growth 
        """
        # Count the number of live refinaries
        buildings_group_name = "live_buildings_" + self.side
        buildings_group     = Director.scene.get_group(buildings_group_name)
        refinary_count      = 0
        maintenance_cost    = 0
        for building in buildings_group:
            if building.name == 'refinery' and building.is_alive:
                refinary_count = refinary_count + 1
            if building.maintence_chrg > 0:
                maintenance_cost = maintenance_cost + building.maintence_chrg 
        
        # Calculate the money-growth
        if refinary_count == 0:
            new_money_growth = 0
        else:
            new_money_growth = \
                (
                    game_options.buildings['refinery']['multiple_effect'] ** 
                    refinary_count - 1
                ) * \
                game_options.buildings['refinery']['money_per_cycle'] 
        # deduct the maintenance cost
        new_money_growth = new_money_growth - maintenance_cost
        self.set_money_growth(new_money_growth)
        
        
    money          =  property(fget=get_money, 
                                    fset=set_money)
    money_growth   =  property(fget=get_money_growth, 
                                    fset=set_money_growth)    
        
class GUIPlayer(Player):
    """ 
        #######################################################################    
        ## This represents the actual player (who plays through the GUI)
        ## User actions are queued in an action queue, and are performed as
        ## the game calls get_action
        ##
        ## Object Variables:        
        ##  action_que      # A queue of user's actions. See above.
        #######################################################################
    """
    
    
    
    #### Methods
    def __init__(self, side, init_money):
        """
            Constructor 
        """
        Player.__init__(self, side, init_money)
        self.action_que = Queue()
        
    def add_action(action):
        """
            When user performs an action via GUI (like buying a building)
            This method is called to add the action to the action-queue
        """
        self.action_que.put(action)
    
    def get_action(self, other_player):
        """
            Return what user have added to action-queue, or None if empty
        """
        if action_que.empty():
            return None
        else:
            return action_que.get()

class AIPlayer(Player):
    """ 
        #######################################################################    
        ## This represents the AI player.
        ## //TODO: Implement later!
        #######################################################################
    """
    