"""
############################################################################
##  units.py
##  
## Contains different unit classes (like tanks, infintry, etc).
############################################################################
"""

import  os
import  math
from    Opioid2D        import *
from    global_options  import game_options
from    targetables     import Targetable
from    utils           import *


class Unit(Sprite, Targetable):
    """
        #######################################################################
        ## This is the superclass of all units (like tanks, infintry, etc)
        ## 
        ## Object Variables:
        ##  type            Type of this unit ('Tank', 'Infintry', etc)
        ##  owner_player    The player who owns this unit
        ##  side            Which side it fights for: 'right' or 'left' 
        ##  current_action  Unit's current action (eg. 'move', 'attack', etc)
        ##                  When the given action is complete, current_action's
        ##                  value is set to 'done'
        ##  current_target  Target for the current action. If the action is
        ##                  'attack', the target should be a targetable 
        ##                  object. If action is 'move', target should be a 
        ##                  (x, y) coordinate. 
        ##  facing          Which way the unit is facing ('up', 'down', 
        ##                  'right', 'left'). It is used to determine which
        ##                  image should be used for unit's sprite (property)
        ##  __facing        private variable that holds the facing value
        ##  speed           Unit's speed (pixels / second)
        ##  hitpoint        Amount of damage the unit causes by each bullet
        ##  fire_range      The radious of the cyrcle unit can attack a target
        #######################################################################
    """
    
    layer       = "units"
    
    def __init__(   self, type, owner_player, max_health, 
                    speed, hitpoint, fire_range):
        """
            Constructor
        """
        Sprite.__init__(self)
        Targetable.__init__(self, max_health)
        self.speed          = speed
        self.hitpoint       = hitpoint
        self.type           = type
        self.owner_player   = owner_player
        self.side           = owner_player.side
        self.current_action = None
        self.current_target = None
        self.fire_range     = fire_range
        self.facing         = opposite_side(self.side)
        self.set(   scale = game_options.unit_scale, 
                    color = game_options.player[self.side]['color']['units'])                
        self.group_name     = "live_units_" + self.side
        self.join_group(self.group_name)

        # Wait a little and start marching
        self.do(    Delay(game_options.unit_action_delay) + \
                    CallFunc(self.__do_init_march))

    def pick_new_action(self):
        """
            The method is called every unit_action_delay, defined in 
            global_options, or when the current action is done.  
            It chooses the unit's next action.
            Currently all units have the same action policy:
            -   They first 'march' a certain distance (unit_initial_march defined 
                in global_options) toward enemy.
            -   If there is an enemy unit on the ground, they move toward it
            -   If there is no enemy unit, they move toward the enemies closest
                building
            -   When they reach the unit/building target, they 'attack' it
            -   If they have choice between attacking a unit and attacking a 
                building (i..e when a building is already within the firing 
                range), they attack the building
        """
        if not self.is_alive :
            return
        # Find the closest enemy building and enemy unit
        closest_enemy_unit, unit_dist   = find_closest_enemy_unit(self)
        closest_enemy_build, build_dist = find_closest_enemy_building(self)
        
        if (    closest_enemy_build is not None and 
                in_fire_range(self, closest_enemy_build)):
            # When we are within fire range of a builidng, we just attack
            self.__do_attack(closest_enemy_build)
        elif (  closest_enemy_unit is not None and 
                in_fire_range(self, closest_enemy_unit)):
            # When no building is in range, but a unit is, we attack the unit
            self.__do_attack(closest_enemy_unit)
        elif closest_enemy_unit is not None:
            # When nothing is in fire range, but an enemy unit is alive, we 
            # move toward it
            self.__do_move_toward(closest_enemy_unit)
        elif closest_enemy_build is not None:
            # When nothing is in fire range, an no enemy unit is alive, we 
            # move toward the closest bulding
            self.__do_move_toward(closest_enemy_build)
        else:
            # When no enemy unit, and no enemy building exists, we wait and 
            # check again later
            self.do(Delay(game_options.unit_action_delay) + \
                    CallFunc(self.pick_new_action))
        

    def __do_init_march(self):
        """
            Carries the initial march.
            Initial march is just moving forward for a certain distance, before
            deciding what we want to do
        """
        init_march = relative_sign_to_side(game_options.unit_initial_march, 
                                           self.side)
        time_required = (   float(math.sqrt(vector_length_sqr(init_march))) / 
                            float(self.speed)                   )
        self.do(MoveDelta(init_march , time_required) + 
                SetAttr(current_action = 'done') + 
                CallFunc(self.pick_new_action))
        
    def __do_move_toward(self, target):
        """
            Moves toward the target.
            
            #TODO: This whole thing is a mess! Needs to be completely rewritten
                
            The first idea is that instead of moving with all kind of weired
            slopes, we want to break the movement into left, right, up, down,
            (Todo: also diagonals: up-right, up-left, down-right, down-left).
            The second idea is that we calculate how much this unit can move
            in unit_action_delay miliseconds, and move that much
            The third idea is that if the target is moving, we don't go toward
            it's current location. We go toward where we think it will be in
            unit_action_delay seconds.
        """
        # This is the max distance we can move within unit_action_delay:
        max_movable_dist        = self.speed * game_options.unit_action_delay
        # This is were the target will be in unit_action_delay:
        projected_target_pos    = calc_projected_pos(
                                    target, game_options.unit_action_delay)
        target_vec = vector_dif(projected_target_pos, self.position)
        # First, choose a moving direction (vertical or horizental)        
        # Our preference is to first become within vertical range of the 
        # target and then move horizentally towards it
        # Todo: add diagonal movement
        if abs(target_vec[1]) > 0.1 * self.fire_range:
            # target is not within vertical range, move vertically
            move_vec    = (0, target_vec[1])
        elif abs(target_vec[0]) > 0:
            # target is not within horizental range, move horizentally
            move_vec    = (target_vec[0], 0)
        
        else:
            # The target is within firing range. We shouldn't have been called
            exit("target is within firing range. Why do you bother me?!")
        # see if we can move the whol distance in one period
        if vector_length_sqr(move_vec) > max_movable_dist**2:
            # no we can't! cut the vector
            move_vec = (
                            (   get_sign(move_vec[0]) * min(abs(move_vec[0]), 
                                max_movable_dist)) ,
                            (   get_sign(move_vec[1]) * min(abs(move_vec[1]), 
                                max_movable_dist))
                        )
            time_required = game_options.unit_action_delay
        else:
            # yes we can, and we don't need the whole period time
            time_required = (   float(math.sqrt(vector_length_sqr(move_vec))) / 
                                float(self.speed))
        self.face_direction(move_vec)
        self.do(MoveDelta(move_vec , time_required) + 
                SetAttr(current_action = 'done') + 
                CallFunc(self.pick_new_action))
        
    def __do_attack(self, target):
        """
            Attackes the target
        """
        target_vec = vector_dif(target.position, self.position)
        self.face_direction(target_vec)
        self._show_anim_attack()
        target.get_hit(self.hitpoint)
        self.do(Delay(game_options.unit_action_delay) + \
                    CallFunc(self.pick_new_action))
        
    
    
    def face_direction(self, dir_vec):
        """
            Given a vector of the direction the unit is moving or shooting, 
            this method will change the facing of the unit, so it 
            (approximately) faces that way
        """
        deg = (((Vector(dir_vec[0],-dir_vec[1])).radial) [0] - 450 ) % 360
     
        if      ((deg >= 0 and deg <= 22.5) or (deg >= 337.5 and deg <= 360)):
            self.facing = 'right'
        elif    (deg > 22.5 and deg <= 67.5) :
            self.facing = 'up_right'
        elif    (deg > 67.5 and deg <= 112.5) :
            self.facing = 'up'
        elif    (deg > 112.5 and deg <= 157.5) :
            self.facing = 'up_left'
        elif    (deg > 157.5 and deg <= 202.5) :
            self.facing = 'left'
        elif    (deg >  202.5 and deg <= 247.5) :
            self.facing = 'down_left'
        elif    (deg >  247.5 and deg <= 292.5) :
            self.facing = 'down'
        elif    (deg >  292.5 and deg <= 337.5) :
            self.facing = 'down_right'
        
    def get_facing(self):
        """
            returns the facing value
        """
        return self.__facing

    def set_facing(self, new_facing):
        """
            Changes the facing value and unit's image 
        """
        self.__facing = new_facing
        image_path =    game_options.gfx_units_path + os.sep + self.type + \
                        "_" + new_facing + ".png"
        self.image = image_path
        # initilize the naimations
        
    facing  =  property(fget=get_facing, 
                        fset=set_facing)
                        
    def die(self):
        """
            Called when the unit is destroyed. Removes unit from player's
            live_units group and deletes the object
        """
        self.leave_group(self.group_name)
        self.do (    AlphaFade(0,secs=0.5) + Delete )
        pass

    
        

class Tank(Unit):
    """
        #######################################################################
        ## This class represents a Tank.
        #######################################################################
    """
    
    def __init__(self, owner_player):
        """
            Constructor
        """
        Unit.__init__(  self, 'tank', owner_player, 
                        game_options.units['tank']['max_helath'], 
                        game_options.units['tank']['speed'],
                        game_options.units['tank']['hitpoint'],
                        game_options.units['tank']['fire_range']    )

    def _show_anim_move(self):
        """
            Shows an animation when the unit is moving.
        """
        # TODO: Implement Pretty Animation
        pass    
        
    def _show_anim_attack(self):
        """
            Shows an animation when the unit is attacking.
        """
        image_pre = game_options.gfx_units_path + os.sep + self.type + \
                        "_" + self.facing 
        attack_anim =  [image_pre + "_atk_1.png", 
                        image_pre + "_atk_2.png", 
                        image_pre + ".png"]
        self.do(Animate(attack_anim, secs=0.4, mode=StopMode))

                
    def _show_anim_hit(self):
        """
            Shows an animation when the unit is hit.
        """
        # TODO: Implement Pretty Animation
        pass
    
    def _show_anim_die(self):
        """
            Shows an animation when the unit is destroyed
        """
        image_pre = game_options.gfx_units_path + os.sep + self.type + \
                        "_" + self.facing 
        die_anim =  []
        for i in range(1, 6):
            die_anim.append(image_pre + "_die_%d.png"%(i))

        self.do(Animate(die_anim, secs=0.5, mode=StopMode))

class Infantry(Unit):
    """
        #######################################################################
        ## This class represents an Infantry.
        #######################################################################
    """
    
    def __init__(self, owner_player):
        """
            Constructor
        """
        Unit.__init__(  self, 'infantry', owner_player, 
                        game_options.units['infantry']['max_helath'], 
                        game_options.units['infantry']['speed'],
                        game_options.units['infantry']['hitpoint'],
                        game_options.units['infantry']['fire_range']    )
    def _show_anim_move(self):
        """
            Shows an animation when the unit is moving.
        """
        # TODO: Implement Pretty Animation
        pass    
        
    def _show_anim_attack(self):
        """
            Shows an animation when the unit is attacking.
        """
        # TODO: Implement Pretty Animation
        pass    

                
    def _show_anim_hit(self):
        """
            Shows an animation when the unit is hit.
        """
        # TODO: Implement Pretty Animation
        pass
    
    def _show_anim_die(self):
        """
            Shows an animation when the unit is destroyed
        """
        # TODO: Implement Pretty Animation
        pass    