"""
############################################################################
##  utils.py
##  
## Contains a number of general utilities used by different classes
############################################################################
"""
import  math
from    sys             import exit, maxint
from    Opioid2D        import *
from    global_options  import game_options


def relative_pos_to_side((x,y), side):
    """
        Given a (x,y) position on the left side, and object's side 
        this function will return the same position if player's side is 'left' 
        and will return the mirrored position if player's side is'right'
    """
    if side == 'left':
        return (x,y)
    elif side == 'right':
        return  calc_mirror_pos((x, y))
    else:
        exit("Unknown side: " + str(side))

def relative_sign_to_side((x,y), side):
    """
        Given a (x,y) vector on the left side, and object's side 
        this function will return the same vector if player's side is 'left'
        and will return the mirrored vector if player's side is 'right'
    """
    if side == 'left':
        return (x,y)
    elif side == 'right':
        return  (-x, y)
    else:
        exit("Unknown side: " + str(side))


def calc_mirror_pos((x, y)):
    """
        Given a (x,y) position on the left side, this function calculates 
        the mirrored positon on the right side
    """
    mirrored_x = game_options.win_width - x
    mirrored_y = y
    return (mirrored_x, mirrored_y)

def opposite_side(side):
    """
        Returns the opposit side of the given side (left <-> right)
    """
    if side == 'left':
        return 'right'
    elif side == 'right':
        return 'left'
    else:
        exit("Unknown side: " + str(side))
        
    
def vector_length_sqr((x,y)):
    """
        Calculates the square of length of the given vector
    """
    return  x*x + y*y

def vector_dif((x1,y1), (x2,y2)):
    """
        Subtracts the two vectors. Returns V1 - V2
    """
    return ((x1 - x2), (y1 - y2))

def find_closest_enemy_unit(unit):
    """
        Given a unit, this method finds the closest live enemy unit to it.
        Returns None if it can't find any units
    """
    enemy_group         = "live_units_" + opposite_side(unit.side)
    enemy_units         = Director.scene.get_group(enemy_group)
    closest_unit        = None
    closest_dist_sqr    = maxint
    for enemy_unit in enemy_units:
        distance_sqr = vector_length_sqr(vector_dif(unit.position,enemy_unit.position))
        if distance_sqr < closest_dist_sqr:
            closest_dist_sqr    = distance_sqr
            closest_unit        = enemy_unit
    return closest_unit, closest_dist_sqr

def find_closest_enemy_building(unit):
    """
        Given a unit, this method finds the closest live enemy building.
        Returns None if it can't find any buildings
    """
    enemy_group         = "live_buildings_" + opposite_side(unit.side)
    enemy_buildings     = Director.scene.get_group(enemy_group)
    closest_building    = None
    closest_dist_sqr    = maxint
    for enemy_building in enemy_buildings:
        distance_sqr = vector_length_sqr(vector_dif(unit.position,
                                            enemy_building.position))
        if distance_sqr < closest_dist_sqr:
            closest_dist_sqr = distance_sqr
            closest_building = enemy_building
    
    return closest_building, closest_dist_sqr

def find_closest_n_enemy_units(unit, range, n):
    """
        Given a unit and a range , this method finds a maximum of n closest 
        live enemy unit to it, within the iven range.
        Returns [] if it can't find any units
    """
    enemy_group = "live_units_" + opposite_side(unit.side)
    enemy_units = Director.scene.get_group(enemy_group)
    if len(enemy_units) < 1:
        return []
    range_sqr   = range ** 2    
    # make a dictionary of enemy units and their distance to us
    in_range_units = {}
    for enemy_unit in enemy_units:    
        distance_sqr = vector_length_sqr(vector_dif(unit.position,enemy_unit.position))
        if distance_sqr < range_sqr:
            in_range_units[enemy_unit] = distance_sqr
    # sort the units based on distance
    in_range_units_sorted = in_range_units.items()
    in_range_units_sorted.sort(key=lambda x: x[1])
    # return the first n units
    closest_n = []
    for pair in in_range_units_sorted[0:n]:
        closest_n.append(pair[0])
    return closest_n
    

def in_fire_range(unit, target):
    """
        Returns true if the target object is within fire range of the unit
    """
    if vector_length_sqr(vector_dif(unit.position,
                                target.position)) < unit.fire_range ** 2:
        return True
    else:
        return False

def in_vertical_range(unit, target):
    """
        Returns true if the target is within vertical ragne of this unit
        The 'vertical range' is 3/4 the firing range on the y-axis
    """
    if vector_dif(unit.position, target.position)[1] <= 0.75 * unit.fire_range:
        return True
    else:
        return False

def calc_projected_pos(unit, delta_t):
    """ 
        Calculates the projected position of the unit, after delta_t seconds
    """
    delta_x = unit.velocity[0] * delta_t
    delta_y = unit.velocity[1] * delta_t
    return unit.position[0] + delta_x, unit.position[1] + delta_y

def get_sign(number):
    """
        Returns the sign of the number, or zero if it is zero
    """
    return cmp(number, 0)